@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2010-2017 The pygit2 contributors
+# Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -40,7 +40,7 @@ def maybe_string(ptr):
     if not ptr:
         return None
 
-    return ffi.string(ptr).decode()
+    return ffi.string(ptr).decode('utf8')
 
 
 class TransferProgress(object):
@@ -128,7 +128,7 @@ class RemoteCallbacks(object):
     def certificate_check(self, certificate, valid, host):
         """
         Certificate callback. Override with your own function to determine
-        whether the accept the server's certificate.
+        whether to accept the server's certificate.
 
         Returns: True to connect, False to abort.
 
@@ -178,12 +178,12 @@ class RemoteCallbacks(object):
     def push_update_reference(self, refname, message):
         """
         Push update reference callback. Override with your own function to
-        report the remote's acceptace or rejection of reference updates.
+        report the remote's acceptance or rejection of reference updates.
 
         refname : str
             The name of the reference (on the remote).
 
-        messsage : str
+        message : str
             Rejection message from the remote. If None, the update was accepted.
         """
 
@@ -210,6 +210,12 @@ class RemoteCallbacks(object):
         self._self_handle = ffi.new_handle(self)
         push_opts.callbacks.payload = self._self_handle
 
+    def _fill_prune_callbacks(self, prune_callbacks):
+        prune_callbacks.update_tips = self._update_tips_cb
+        # We need to make sure that this handle stays alive
+        self._self_handle = ffi.new_handle(self)
+        prune_callbacks.payload = self._self_handle
+
     # These functions exist to be called by the git_remote as
     # callbacks. They proxy the call to whatever the user set
 
@@ -238,7 +244,7 @@ class RemoteCallbacks(object):
             return 0
 
         try:
-            s = ffi.string(string, length).decode()
+            s = ffi.string(string, length).decode('utf-8')
             progress(s)
         except Exception as e:
             self._stored_exception = e
@@ -408,6 +414,17 @@ class Remote(object):
 
         return TransferProgress(C.git_remote_stats(self._remote))
 
+    def prune(self, callbacks=None):
+        """Perform a prune against this remote."""
+
+        remote_callbacks = ffi.new('git_remote_callbacks *')
+        C.git_remote_init_callbacks(remote_callbacks, C.GIT_REMOTE_CALLBACKS_VERSION)
+        if callbacks is None:
+            callbacks = RemoteCallbacks()
+        callbacks._fill_prune_callbacks(remote_callbacks)
+        err = C.git_remote_prune(self._remote, remote_callbacks)
+        check_error(err)
+
     @property
     def refspec_count(self):
         """Total number of refspecs in this remote"""
@@ -445,7 +462,7 @@ class Remote(object):
         error or unpack failure.
 
         When the remote has a githook installed, that denies the reference this
-        function will return successfully. Thus it is stronly recommended to
+        function will return successfully. Thus it is strongly recommended to
         install a callback, that implements
         :py:meth:`RemoteCallbacks.push_update_reference` and check the passed
         parameters for successfull operations.
@@ -508,6 +525,13 @@ def get_credentials(fn, url, username, allowed):
         name, = credential_tuple
         err = C.git_cred_username_new(ccred, to_bytes(name))
 
+    elif cred_type == C.GIT_CREDTYPE_SSH_MEMORY:
+        name, pubkey, privkey, passphrase = credential_tuple
+        if pubkey is None and privkey is None:
+            raise TypeError("SSH keys from memory are empty")
+        err = C.git_cred_ssh_key_memory_new(ccred, to_bytes(name),
+                                            to_bytes(pubkey), to_bytes(privkey),
+                                            to_bytes(passphrase))
     else:
         raise TypeError("unsupported credential type")
 
